"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseFunction = exports.parseWhen = exports.parseSetters = exports.parseValidators = exports.parseValidator = exports.simpleClone = exports.getKey = exports.defineValue = exports.isArray = exports.isDefined = exports.isObject = exports.isRegExp = exports.isFunction = exports.isString = exports.NOOP = exports.PREFIX_HAS = exports.PREFIX_IS = exports.UNDEFINED = exports.defineProperty = void 0;

var _makeArray = _interopRequireDefault(require("make-array"));

var _error = require("./error");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  defineProperty
} = Object;
exports.defineProperty = defineProperty;
const UNDEFINED = void 0;
exports.UNDEFINED = UNDEFINED;
const PREFIX_IS = 'is';
exports.PREFIX_IS = PREFIX_IS;
const PREFIX_HAS = 'has';
exports.PREFIX_HAS = PREFIX_HAS;

const NOOP = () => {};

exports.NOOP = NOOP;

const isString = subject => typeof subject === 'string';

exports.isString = isString;

const isFunction = subject => typeof subject === 'function';

exports.isFunction = isFunction;

const isRegExp = subject => !!subject && typeof subject.test === 'function';

exports.isRegExp = isRegExp;

const isObject = subject => !!subject && Object(subject) === subject;

exports.isObject = isObject;

const isDefined = subject => subject !== UNDEFINED;

exports.isDefined = isDefined;
const isArray = Array.isArray;
exports.isArray = isArray;

const defineValue = (object, key, value) => defineProperty(object, key, {
  value
});

exports.defineValue = defineValue;

const hypenate = key => key[0].toUpperCase() + key.slice(1);

const getKey = (key, prefix) => prefix + hypenate(key);

exports.getKey = getKey;

const simpleClone = object => {
  return Object.assign(Object.create(null), object);
}; // See "schema design"


exports.simpleClone = simpleClone;

const parseValidator = validator => {
  if (isFunction(validator)) {
    return validator;
  }

  if (isRegExp(validator)) {
    return v => validator.test(v);
  }

  throw (0, _error.error)('INVALID_VALIDATOR');
};

exports.parseValidator = parseValidator;

const parseValidators = validators => {
  if (!validators) {
    return;
  }

  return (0, _makeArray.default)(validators).map(parseValidator);
};

exports.parseValidators = parseValidators;

const parseSetters = setters => {
  if (!setters) {
    return;
  }

  return (0, _makeArray.default)(setters).map(setter => {
    if (isFunction(setter)) {
      return setter;
    }

    throw (0, _error.error)('INVALID_SETTER');
  });
};

exports.parseSetters = parseSetters;

const parseWhen = when => {
  if (isFunction(when)) {
    return when;
  }

  if (when === false) {
    return () => false;
  }
};

exports.parseWhen = parseWhen;

const parseFunction = subject => {
  if (isFunction(subject)) {
    return subject;
  }

  if (isDefined(subject)) {
    return () => subject;
  }
};

exports.parseFunction = parseFunction;