"use strict";

// Used for JsFiddle
// https://github.com/kaelzhang/skema
////////////////////////////////////////////////////////
function _log(className) {
  var div = document.createElement('div');
  div.className = className ? 'log ' + className : 'log';
  var contents = [];
  var length = arguments.length;
  var i = 1;

  while (i < length) {
    var arg = arguments[i++];
    contents.push(arg === void 0 ? 'undefined' : arg === null ? 'null' : JSON.stringify(arg, null, 2));
  }

  div.textContent = contents.join(' ');
  document.body.appendChild(div);
}

function success() {
  _log.apply(null, ['success'].concat(Array.prototype.slice.call(arguments)));
}

function fail() {
  _log.apply(null, ['fail'].concat(Array.prototype.slice.call(arguments)));
}

function log() {
  _log.apply(null, ['info'].concat(Array.prototype.slice.call(arguments)));
}

function throws(name, func) {
  if (typeof name === 'function') {
    func = name;
    name = '';
  }

  try {
    func();
  } catch (e) {
    name ? fail(name, e.message, e) : fail(e.message, e);
    return;
  }
}

function addCSS(css) {
  var style = document.createElement('style');
  style.type = 'text/css';

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }

  var head = document.head || document.getElementByTagName('head')[0];
  head.appendChild(style);
}

addCSS("\nbody {\n  margin: 1px;\n  padding: 20px;\n  white-space: pre;\n  font: 12px monospace;\n  line-height: 18px;\n  background: transparent;\n}\n\n.log {\n  min-height: 18px;\n  color: #f7f7f8;\n}\n\n.success {\n  color: #6ce890;\n}\n\n.fail {\n  color: #ed6e55;\n}\n");
